/***
Copyright (C) JH Zhao 08-06-2000 IoP
Power of multiallelic TDT, by allele-wise test

Reference

Sham PC & D Curtis (1995). An extended
transmission/disequilibrium test (TDT) for multi-allele
marker loci.  Ann Hum Genet 59, 323-336

***/

options nocenter ps=300;
title;

proc iml;

/*f - degrees of freedom*/
/*alpha - type I error*/
/*beta  - type II error = 1-power*/

start calcNCP(f) global (alpha,beta);

c=cinv(1-alpha,f);
NCP=cnonct(c,f,beta);
p=probchi(c,f,c);

return (NCP);

finish calcNCP;

/*m - number of alleles*/
/*q - disease allele frequencies*/
/*f - penetrances, f[1]>=f[2]>=f[3]*/
/*p - marker allele frequencies*/
/*h - disease-marker haplotypes*/
/*t - recombination fraction*/

start ETDTx2(m) global (q,f,p,h,t);
q1=q[1];
q2=q[2];
f11=f[1];
f12=f[2];
f22=f[3];

print 'disease model, allele/haplotype frequencies', q f p[format=6.4] h[format=6.4];

/*e - LD parameter*/
e=j(2,m,0);

do s=1 to 2;
   do i=1 to m;
      e[s,i]=h[(s-1)*m+i]/q[s]/p[i];
   end;
end;

print 'linkage disequilibrium parameter', e[format=8.4];

/*transimission inequilibrium parameter*/
d=j(m,m,0);

B=(q1*(q1*(f11-f12)+q2*(f12-f22)))/(q1**2*f11+2*q1*q2*f12+q2**2*f22);

do i=1 to m;
   e1i=e[1,i];
   do j=1 to m;
      e1j=e[1,j];
      d[i,j]=1+B*((e1i-1)+t*(e1j-e1i));
   end;
end;

print 'transmission inequility parameter', d[format=8.4];

/*transmission table*/
table=j(m,m,0);

do i=1 to m;
   do j=1 to m;
      table[i,j]=p[i]*p[j]*d[i,j];
   end;
end;

print 'transmission table', table[format=8.4];

/*chi2 under alternative*/
/*l0s - null*/
/*l1s - alternative*/
l0s=0;
l1s=0;
do j=1 to m;
   djj=d[j,j];
   do i=1 to j-1;
      dii=d[i,i];
      diijj=djj+dii;
      l0s=l0s+p[i]*p[j]*(dii+djj)*log(1/2);
      l1s=l1s+p[i]*p[j]*(dii*log(dii/diijj)+djj*log(djj/diijj));
   end;
end;

x2s=2*(l1s-l0s);
print 'L1*=' l1s 'L0*=' l0s;

return (x2s);

finish ETDTx2;


/*Disease locus, Alzheimer's model*/
q=j(2,1,0);
q[1]=0.13;
q[2]=1-q[1];
f=j(3,1,0);
f[1]=0.8;
f[2]=0.2;
f[3]=0.05;

/*Marker locus, DXS548*/
m=7;
p=j(m,1,0);
fragileX={0 42 32 1 1 29 1, 2 117 23 1 1 15 2};
alleles=fragileX[+];
do i=1 to m;
   p[i]=(fragileX[1,i]+fragileX[2,i])/alleles;
end;

/*haplotype frequencies and recombination fraction*/
h=j(2*m,1,0);
do i=1 to m;
   h[i]=fragileX[1,i]/alleles;
   h[i+m]=fragileX[2,i]/alleles;
end;
t=0;

/*setting for power caculation*/
alpha=0.01;
beta=0.10;
df=m-1;

/*obtain sample size and power*/
x2=ETDTx2(m);
NCP=calcNCP(df);
n=NCP/x2;
p=probchi(cinv(1-alpha,df),df,x2);

print 'alpha=' alpha 'beta=' beta 'theta=' t 'power=' p;
print 'x2=' x2 '' 'NCP=' ncp 'sample size=' n;

quit;
