function X=tdtsimse(T,n)

% function X=tdtsimse(T,n)
%
% Monte Carlo simulation for the Spielman-Ewens multiallelic 
% TDT test.  ALL n chi-square values are stored in X.
% 
% T = Table of transmitted (rows) and not transmitted (columns)
% n = number of resamples


% Set diagonal elements of table to zero.

T=T-diag(diag(T));


% Find alleles that do not appear in transmitting heterozygous
% parents and remove the appropriate rows and columns of T.

keep=find(sum(T)+sum(T'));
T=T(keep,keep);


% Find non-zero off-diagonal elements of table.

[i,j]=find(tril(T)+triu(T)');


% Count number of alleles.

m=size(T,1);


% Count number of different heterozygotes.

NH=length(i);


% Count number of each heterozygote.

for k=1:NH, C(k,1)=T(i(k),j(k))+T(j(k),i(k)); end


% Simulate tables and compute TDT chi-square statistics.

T0=T;
for K=1:n,
   for L=1:NH, T(i(L),j(L))=sum(rand(C(L),1)<.5); T(j(L),i(L))=C(L)-T(i(L),j(L)); end
   X(K,1)=((m-1)/m)*sum(((sum(T)-sum(T')).^2)./(sum(T)+sum(T')));
end
MCp=mean(X>=((m-1)/m)*sum(((sum(T0)-sum(T0')).^2)./(sum(T0)+sum(T0'))))
