function X=tdtsimst(T,n)

% function X=tdtsimst(T,n)
%
% Monte Carlo simulation for the Stuart (1955) multiallelic TDT test
% (after Sham, 9/1997, AJHG).  ALL n chi-square values are stored in X.
% 
% T = Table of transmitted (rows) and not transmitted (columns)
% n = number of MC runs


% Set diagonal elements of table to zero.

T=T-diag(diag(T));


% Count number of alleles.

m=size(T,1);


% Produce inverse (IV) of variance-covariance matrix (V).  This is 
% constant across repeated samples in the Monte Carlo simulation.

V=diag(sum(T')'+sum(T)')-tril(T)-triu(T)'-tril(T)'-triu(T);
IV=inv(V(1:(m-1),1:(m-1)));


% Find non-zero off-diagonal elements of table.

[i,j]=find(tril(T)+triu(T)');


% Count number of different heterozygotes.

NH=length(i);


% Count number of each heterozygote.

for k=1:NH, C(k,1)=T(i(k),j(k))+T(j(k),i(k)); end


% Simulate tables and compute TDT chi-square statistics.

T0=T;
for K=1:n,
   for L=1:NH, T(i(L),j(L))=sum(rand(C(L),1)<.5); T(j(L),i(L))=C(L)-T(i(L),j(L)); end
   d=sum(T(1:(m-1),1:(m-1))')'-sum(T(1:(m-1),1:(m-1)))';
   X(K,1)=d'*IV*d;
end
d0=sum(T0(1:(m-1),1:(m-1))')'-sum(T0(1:(m-1),1:(m-1)))';
MCp=mean(X>=(d0'*IV*d0))
