title;
options ps=5000 ls=120 compress=no nocenter;

%macro bt(data,n);
/*
Bradley-Terry model, adapted for multiallelic TDT

the variables in the data set have to be x1-xn

%bt(data,n) will invoke the macro.

JH Zhao 14/5/1999 IOP, 5/1/2004 UCL
*/
data _bt_;
  set &data;
  array x {&n} x1-x&n;
  array allele {&n} y1-y&n;
  do i=1 to &n; allele{i}=0; end;
  y=1;
  do i=1 to &n;
     allele{_n_}=1;
     allele{i}=-1;
     count=x{i};
     if _n_ ne i then output;
     allele{i}=0;
  end;
  keep y count y1-y&n;
run;

proc logistic data=_bt_;
  freq count;
  model y=y1-y&n / noint;
  output out=out p=p;
  run;
   
* proc print data=_bt_ noobs;
*   var count y1-y&n;
* run;

%mend;

%macro toETDT(data);   
/*Create the cou format for ETDT/LRTDT*/
/*Only variables x1-xn are allowed in &data*/
/*Add n to the very top of the output and drop variable names*/
/*JH Zhao 15/5/1999*/
proc iml;
   use &data;
   read all into a;
   n=nrow(a);
   nn1=n*(n-1)/2;   
   c=j(nn1,1,0);
   t=j(nn1,1,0);
   i1=j(nn1,1,0);
   i2=j(nn1,1,0);
   ij=j(nn1,n,0);
   vi=j(1,n,0);   
   l=0;
   do i=1 to n-1;
      do j=i+1 to n; 
	 l=l+1;
	 c[l]=a[i,j]+a[j,i];
	 t[l]=a[i,j];
	 i1[l]=i;
	 i2[l]=j;
	 ij[l,i]=1;	 
	 ij[l,j]=-1;
      end;
      vi[i]=i;      
   end;
   close a;
   all=i1||i2||c||t||ij;
   id={i1 i2 c t};
   yind=char(vi,2);
   vname=concat('y',left(yind));
   id=id||vname;   
   create _etdt_ from all[colname=id];
   append from all;
   
proc print data=_etdt_ noobs;
run;

%mend;

/*Copeman et al (1995) Nat Genet 9: 80-5*/
/*Curnow et al (1998) Appl Statist 47: 63-76*/

data a;
  input x1-x12;
cards;
0 0  0  2  0 0  0  0  0  0  0  0
0 0  1  3  0 0  0  2  3  0  0  0
2 3 26 35  7 0  2 10 11  3  4  1
2 3 22 26  6 2  4  4 10  2  2  0
0 1  7 10  2 0  0  2  2  1  1  0
0 0  1  4  0 1  0  1  0  0  0  0
0 2  5  4  1 1  0  0  0  2  0  0
0 0  2  6  1 0  2  0  2  0  0  0
0 3  6 19  6 0  0  2  5  3  0  0
0 0  3  1  1 0  0  0  1  0  0  0
0 0  0  2  0 0  0  0  0  0  0  0
0 0  1  0  0 0  0  0  0  0  0  0
;

%bt(a,12);
%toETDT(a);
   
data b;
   do i=1 to 12;
      do j=1 to 12;
	 input w @@;
	 output;
	 end;
      end;
   cards;
0 0  0  2  0 0  0  0  0  0  0  0
0 0  1  3  0 0  0  2  3  0  0  0
2 3 26 35  7 0  2 10 11  3  4  1
2 3 22 26  6 2  4  4 10  2  2  0
0 1  7 10  2 0  0  2  2  1  1  0
0 0  1  4  0 1  0  1  0  0  0  0
0 2  5  4  1 1  0  0  0  2  0  0
0 0  2  6  1 0  2  0  2  0  0  0
0 3  6 19  6 0  0  2  5  3  0  0
0 0  3  1  1 0  0  0  1  0  0  0
0 0  0  2  0 0  0  0  0  0  0  0
0 0  1  0  0 0  0  0  0  0  0  0
;
proc freq;
   weight w;   
   table i*j/agree noprint;
   run;
   
options missing=' ';
proc tabulate f=3. fc=' ----------' missing;
   class i j;
   freq w;
   table i all,j all*f=6. /rts=10;
   keylabel n=' ' all='total';
   run;
   
